package at.bestsolution.rsd.rt;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Spliterator;
import java.util.Spliterators;
import java.util.function.BiPredicate;
import java.util.function.Function;
import java.util.function.ToIntFunction;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

final class ArrayImpl<O> implements Array<O>, Serializable {
	/**
	 *
	 */
	private static final long serialVersionUID = 1L;

	private Object[] array;

	ArrayImpl( Object[] array, boolean copy ) {
		if( copy ) {
			this.array = new Object[array.length];
			System.arraycopy( array, 0, this.array, 0, array.length );			
		} else {
			this.array = array;
		}
	}

	@Override
	public int length() {
		return array.length;
	}

	@Override
	public boolean isEmpty() {
		return array.length == 0;
	}

	@SuppressWarnings( "unchecked" )
	@Override
	public O get( int index ) {
		return (O) this.array[index];
	}

	@SuppressWarnings( "unchecked" )
	@Override
	public <S> S[] toArray( S[] target ) {
		if( target.length == array.length ) {
			System.arraycopy( array, 0, target, 0, target.length );
			return target;
		}
		return (S[]) Arrays.copyOf( array, array.length, target.getClass() );
	}

	@Override
	public Stream<O> stream() {
		return StreamSupport.stream( Spliterators.spliterator( array,
				Spliterator.ORDERED | Spliterator.IMMUTABLE ), false );
	}

	@Override
	public <R> List<R> mapTo( Function<O, R> map ) {
		List<R> rv = new ArrayList<>( array.length );
		for( int i = 0; i < array.length; i++ ) {
			rv.add( map.apply( get(i) ) );
		}
		return Collections.unmodifiableList( rv );
	}

	@SuppressWarnings( "unchecked" )
	@Override
	public List<O> toList() {
		return (List<O>) Arrays.asList( array );
	}

	@Override
	public int deepHashCode( ToIntFunction<O> function ) {
		int result = 1;
		for( int i = 0; i < this.array.length; i++ ) {
			result += 31 * result + function.applyAsInt( get(i) );
		}
		return result;
	}

	@Override
	public boolean deepEquals( Array<O> obj, BiPredicate<O, O> function ) {
		if( obj == null || obj.length() != length() ) {
			return false;
		}

		for( int i = 0; i < array.length; i++ ) {
			if( !function.test( obj.get( i ), get(i) ) ) {
				return false;
			}
		}
		return true;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Arrays.deepHashCode( array );
		return result;
	}

	@Override
	public boolean equals( Object obj ) {
		if( this == obj )
			return true;
		if( obj == null )
			return false;

		if( obj instanceof ArrayImpl ) {
			ArrayImpl<?> other = (ArrayImpl<?>) obj;
			return Arrays.equals( array, other.array );
		} else {
			if( obj instanceof Array ) {
				Array<?> other = (Array<?>) obj;
				if( length() == other.length() ) {
					return Arrays.equals( array, other.toArray( new Object[0] ) );
				}
			}
		}
		return false;
	}

	@Override
	public Iterator<O> iterator() {
		return new Iterator<O>() {
			private int idx = 0;

			@Override
			public boolean hasNext() {
				return idx < array.length;
			}

			@Override
			public O next() {
				if( idx >= array.length ) {
					throw new NoSuchElementException();
				}
				return get(idx++);
			}

		};
	}
}