package at.bestsolution.rsd.rt;

import java.util.HashMap;
import java.util.Map;
import java.util.ServiceLoader;

import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

@Component
public class BuilderFactoryComponent {
	private Map<Class<?>, BuilderFactory> typeMap = new HashMap<>();
	private Map<Class<?>, BuilderFactory> builderTypeMap = new HashMap<>();
	private static volatile BuilderFactoryComponent instance;

	public static BuilderFactoryComponent getInstance() {
		if( instance == null ) {
			synchronized (BuilderFactoryComponent.class) {
				if( instance == null ) {
					ServiceProviderSystem providerSystem = ServiceLoader.load(ServiceProviderSystem.class).iterator().next();
					instance = providerSystem.lookupServices(BuilderFactoryComponent.class).get(0);
				}
			}
		}
		return instance;
	}

	@Reference(cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY)
	public void registerBuilderFactory(BuilderFactory factory) {
		synchronized (builderTypeMap) {
			factory.builderTypes().forEach( t -> builderTypeMap.put(t, factory));
		}
		synchronized (typeMap) {
			factory.types().forEach( t -> typeMap.put(t, factory));
		}
	}

	public void unregisterBuilderFactory(BuilderFactory factory) {
		synchronized (builderTypeMap) {
			factory.builderTypes().forEach( k -> builderTypeMap.remove(k));
		}

		synchronized (typeMap) {
			factory.types().forEach( k -> typeMap.remove(k));
		}
	}

	public boolean hasBuilder(Class<?> clazz) {
		return builderTypeMap.containsKey(clazz);
	}

	public boolean hasBuilderForType(Class<?> clazz) {
		return typeMap.containsKey(clazz);
	}

	public <T> T createBuilder(Class<T> clazz) {
		BuilderFactory factory = builderTypeMap.get(clazz);
		if( factory != null ) {
			return factory.createBuilder(clazz);
		}
		throw new IllegalArgumentException("The builder '"+clazz+"' is not known.");
	}

	public <T> T createBuilderForType(Class<?> clazz) {
		BuilderFactory factory = typeMap.get(clazz);
		if( factory != null ) {
			return factory.createBuilder(clazz);
		}
		throw new IllegalArgumentException("No builder for '"+clazz+"' is found");
	}
}
