package at.bestsolution.rsd.rt;

import java.io.Serializable;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.function.BiPredicate;
import java.util.function.Function;
import java.util.function.ToIntFunction;
import java.util.stream.Stream;

final class EmptyArray<T> implements Array<T>, Serializable {

	private static final long serialVersionUID = 1L;
	
	static final EmptyArray<?> SELF = new EmptyArray<>();

	@Override
	public Iterator<T> iterator() {
		return new Iterator<T>() {

			@Override
			public boolean hasNext() {
				return false;
			}

			@Override
			public T next() {
				throw new NoSuchElementException();
			}
		};
	}

	@Override
	public int length() {
		return 0;
	}

	@Override
	public boolean isEmpty() {
		return true;
	}

	@SuppressWarnings("unchecked")
	@Override
	public T[] toArray(Object[] target) {
		return (T[]) java.lang.reflect.Array.newInstance(target.getClass().getComponentType(), 0);
	}

	@Override
	public T get(int index) {
		throw new ArrayIndexOutOfBoundsException(index);
	}

	@Override
	public Stream<T> stream() {
		return Stream.empty();
	}
	
	@Override
	public <R> List<R> mapTo(Function<T, R> map) {
		return Collections.emptyList();
	}
	
	@Override
	public int deepHashCode(ToIntFunction<T> function) {
		return 0;
	}
	
	@Override
	public boolean deepEquals(Array<T> array, BiPredicate<T, T> function) {
		if( array == null ) {
			return false;
		}
		return array.isEmpty();
	}
	
	@Override
	public int hashCode() {
		return 0;
	}
	
	@Override
	public boolean equals(Object obj) {
		return obj instanceof Array && ((Array<?>)obj).isEmpty();
	}
}
