package at.bestsolution.rsd.rt;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.function.BiPredicate;
import java.util.function.Function;
import java.util.function.ToIntFunction;
import java.util.stream.Stream;

import at.bestsolution.rsd.rt.Array.DoubleArray;

final class EmptyDoubleArray implements DoubleArray {
	static final EmptyDoubleArray SELF = new EmptyDoubleArray();
	
	@Override
	public int length() {
		return 0;
	}

	@Override
	public boolean isEmpty() {
		return true;
	}

	@SuppressWarnings("unchecked")
	@Override
	public <S> S[] toArray(S[] target) {
		return (S[]) java.lang.reflect.Array.newInstance(target.getClass().getComponentType(), 0);
	}

	@Override
	public Double get(int index) {
		throw new ArrayIndexOutOfBoundsException(index);
	}

	@Override
	public Stream<Double> stream() {
		return Stream.empty();
	}

	@Override
	public int deepHashCode(ToIntFunction<Double> function) {
		return 0;
	}

	@Override
	public boolean deepEquals(Array<Double> array, BiPredicate<Double, Double> function) {
		if( array == null ) {
			return false;
		}
		return array.isEmpty();
	}

	@Override
	public <R> List<R> mapTo(Function<Double, R> map) {
		return Collections.emptyList();
	}

	@Override
	public Iterator<Double> iterator() {
		return new Iterator<Double>() {

			@Override
			public boolean hasNext() {
				return false;
			}

			@Override
			public Double next() {
				throw new NoSuchElementException();
			}
		};
	}

	@Override
	public double getAsDouble(int index) {
		throw new ArrayIndexOutOfBoundsException(index);
	}

	@Override
	public double[] toDoubleArray() {
		return new double[0];
	}

	@Override
	public int deepHashCode() {
		return 0;
	}

	@Override
	public boolean deepEquals(DoubleArray array) {
		if( array == null ) {
			return false;
		}
		return array.isEmpty();
	}

	@Override
	public int hashCode() {
		return 0;
	}
	
	@Override
	public boolean equals(Object obj) {
		return obj instanceof Array && ((Array<?>)obj).isEmpty();
	}
}
