package at.bestsolution.rsd.rt;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.function.BiPredicate;
import java.util.function.Function;
import java.util.function.ToIntFunction;
import java.util.stream.Stream;

import at.bestsolution.rsd.rt.Array.IntArray;

final class EmptyIntArray implements IntArray {
	static final EmptyIntArray SELF = new EmptyIntArray();
	
	@Override
	public int length() {
		return 0;
	}

	@Override
	public boolean isEmpty() {
		return true;
	}

	@SuppressWarnings("unchecked")
	@Override
	public <S> S[] toArray(S[] target) {
		return (S[]) java.lang.reflect.Array.newInstance(target.getClass().getComponentType(), 0);
	}

	@Override
	public Integer get(int index) {
		throw new ArrayIndexOutOfBoundsException(index);
	}

	@Override
	public Stream<Integer> stream() {
		return Stream.empty();
	}

	@Override
	public int deepHashCode(ToIntFunction<Integer> function) {
		return 0;
	}

	@Override
	public boolean deepEquals(Array<Integer> array, BiPredicate<Integer, Integer> function) {
		if( array == null ) {
			return false;
		}
		return array.isEmpty();
	}

	@Override
	public <R> List<R> mapTo(Function<Integer, R> map) {
		return Collections.emptyList();
	}

	@Override
	public Iterator<Integer> iterator() {
		return new Iterator<Integer>() {

			@Override
			public boolean hasNext() {
				return false;
			}

			@Override
			public Integer next() {
				throw new NoSuchElementException();
			}
		};
	}

	@Override
	public int getAsInt(int index) {
		throw new ArrayIndexOutOfBoundsException(index);
	}

	@Override
	public int[] toIntArray() {
		return new int[0];
	}

	@Override
	public int deepHashCode() {
		return 0;
	}

	@Override
	public boolean deepEquals(IntArray array) {
		if( array == null ) {
			return false;
		}
		return array.isEmpty();
	}

	@Override
	public int hashCode() {
		return 0;
	}
	
	@Override
	public boolean equals(Object obj) {
		return obj instanceof Array && ((Array<?>)obj).isEmpty();
	}
}
