package at.bestsolution.rsd.rt;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.function.BiPredicate;
import java.util.function.Function;
import java.util.function.ToIntFunction;
import java.util.stream.Stream;

import at.bestsolution.rsd.rt.Array.LongArray;

final class EmptyLongArray implements LongArray {
	static final EmptyLongArray SELF = new EmptyLongArray();
	
	@Override
	public int length() {
		return 0;
	}

	@Override
	public boolean isEmpty() {
		return true;
	}

	@SuppressWarnings("unchecked")
	@Override
	public <S> S[] toArray(S[] target) {
		return (S[]) java.lang.reflect.Array.newInstance(target.getClass().getComponentType(), 0);
	}

	@Override
	public Long get(int index) {
		throw new ArrayIndexOutOfBoundsException(index);
	}

	@Override
	public Stream<Long> stream() {
		return Stream.empty();
	}

	@Override
	public int deepHashCode(ToIntFunction<Long> function) {
		return 0;
	}

	@Override
	public boolean deepEquals(Array<Long> array, BiPredicate<Long, Long> function) {
		if( array == null ) {
			return false;
		}
		return array.isEmpty();
	}

	@Override
	public <R> List<R> mapTo(Function<Long, R> map) {
		return Collections.emptyList();
	}

	@Override
	public Iterator<Long> iterator() {
		return new Iterator<Long>() {

			@Override
			public boolean hasNext() {
				return false;
			}

			@Override
			public Long next() {
				throw new NoSuchElementException();
			}
		};
	}

	@Override
	public long getAsLong(int index) {
		throw new ArrayIndexOutOfBoundsException(index);
	}

	@Override
	public long[] toLongArray() {
		return new long[0];
	}

	@Override
	public int deepHashCode() {
		return 0;
	}

	@Override
	public boolean deepEquals(LongArray array) {
		if( array == null ) {
			return false;
		}
		return array.isEmpty();
	}

	@Override
	public int hashCode() {
		return 0;
	}
	
	@Override
	public boolean equals(Object obj) {
		return obj instanceof Array && ((Array<?>)obj).isEmpty();
	}
}
