package at.bestsolution.rsd.rt;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.function.BiPredicate;
import java.util.function.Function;
import java.util.function.ToIntFunction;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import at.bestsolution.rsd.rt.Array.IntArray;

final class IntArrayImpl implements IntArray, Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private int[] array;

	public IntArrayImpl(int[] array) {
		this.array = Arrays.copyOf(array, array.length);
	}

	@Override
	public int length() {
		return array.length;
	}

	@Override
	public boolean isEmpty() {
		return array.length == 0;
	}

	@Override
	public Integer get(int index) {
		return this.array[index];
	}

	@Override
	public int getAsInt(int index) {
		return this.array[index];
	}

	@Override
	public int[] toIntArray() {
		return Arrays.copyOf(this.array, this.array.length);
	}

	@Override
	public int deepHashCode() {
		return Arrays.hashCode(this.array);
	}

	@Override
	public int deepHashCode(ToIntFunction<Integer> function) {
		int result = 1;
		for (int i = 0; i < this.array.length; i++) {
			result += 31 * result + function.applyAsInt(this.array[i]);
		}
		return result;
	}

	@Override
	public boolean deepEquals(IntArray obj) {
		if( obj == null ) {
			return false;
		}
		
		if( obj instanceof IntArrayImpl ) {
			return Arrays.equals(this.array,((IntArrayImpl)obj).array);
		}
		
		return Arrays.equals(this.array,obj.toIntArray());
	}

	@Override
	public boolean deepEquals(Array<Integer> obj, BiPredicate<Integer, Integer> function) {
		if( obj == null || obj.length() != length() ) {
			return false;
		}
		for( int i = 0; i < array.length; i++ ) {
			if( ! function.test(obj.get(i), this.array[i]) ) {
				return false; 
			}
		}
		return true;
	}

	@SuppressWarnings("unchecked")
	@Override
	public <S> S[] toArray(S[] target) {
		if (target.length == array.length) {
			System.arraycopy(array, 0, target, 0, target.length);
			return target;
		}
		S[] rv = (S[]) java.lang.reflect.Array.newInstance(target.getClass().getComponentType(), array.length);
		System.arraycopy(array, 0, rv, 0, rv.length);
		return rv;
	}

	@Override
	public Stream<Integer> stream() {
		return IntStream.of(array).mapToObj(Integer::valueOf);
	}

	@Override
	public <R> List<R> mapTo(Function<Integer, R> map) {
		List<R> rv = new ArrayList<>(array.length);
		for (int i = 0; i < array.length; i++) {
			rv.add(map.apply(array[i]));
		}
		return Collections.unmodifiableList(rv);
	}

	@Override
	public Iterator<Integer> iterator() {
		return new Iterator<Integer>() {
			private int idx = 0;

			@Override
			public boolean hasNext() {
				return idx < array.length;
			}

			@Override
			public Integer next() {
				if (idx >= array.length) {
					throw new NoSuchElementException();
				}
				return array[idx++];
			}

		};
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Arrays.hashCode(array);
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;

		if (obj instanceof IntArrayImpl) {
			IntArrayImpl other = (IntArrayImpl) obj;
			return Arrays.equals(array, other.array);
		} else {
			if (obj instanceof IntArray) {
				IntArray other = (IntArray) obj;
				if (length() == other.length()) {
					return Arrays.equals(array, other.toIntArray());
				}
			} else if (obj instanceof Array) {
				Array<?> other = (Array<?>) obj;
				if (length() == other.length()) {
					return Arrays.equals(toArray(new Object[0]), other.toArray(new Object[0]));
				}
			}
			return false;
		}
	}
}