package at.bestsolution.rsd.rt;

import java.util.concurrent.CopyOnWriteArrayList;
import java.util.function.Consumer;

import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import at.bestsolution.rsd.rt.interceptor.RemoteCallInterceptor;
import at.bestsolution.rsd.rt.interceptor.RemoteResultInterceptor;
import at.bestsolution.rsd.rt.interceptor.RemoteResultInterceptor.ResultContext;

@Component
public class InterceptorRegistry {
	private CopyOnWriteArrayList<RemoteResultInterceptor> resultInterceptors = new CopyOnWriteArrayList<>();
	private CopyOnWriteArrayList<RemoteCallInterceptor> callInterceptors = new CopyOnWriteArrayList<>();

	@Reference(cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY)
	public void registerRemoteCallInterceptor(RemoteCallInterceptor interceptor) {
		callInterceptors.add(interceptor);
	}

	public void unregisterRemoteCallInterceptor(RemoteCallInterceptor interceptor) {
		callInterceptors.remove(interceptor);
	}

	@Reference(cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY)
	public void registerRemoteResultInterceptor(RemoteResultInterceptor interceptor) {
		resultInterceptors.add(interceptor);
	}

	public void unregisterRemoteResultInterceptor(RemoteResultInterceptor interceptor) {
		resultInterceptors.remove(interceptor);
	}
	
	public void forEachCallInterceptor(Consumer<RemoteCallInterceptor> consumer) {
		callInterceptors.forEach(consumer);
	}
	
	@SuppressWarnings("unchecked")
	public <T> T processResult(T value, Throwable t, ResultContext context) {
		T localVal = value;
		for( RemoteResultInterceptor i : resultInterceptors ) {
			localVal = (T) i.process(context, localVal, t);
		}
		return value;
	}
}
