package at.bestsolution.rsd.rt;

import java.util.Objects;

public interface Ref<T> {
	public String id();
	public String etag();
	
	public Class<T> _type();
	
	public default int deepHashCode() {
		int result = 1;
		result = 31 * result + Objects.hashCode(_type());
		result = 31 * result + Objects.hashCode(id());
		result = 31 * result + Objects.hashCode(etag());
		return result;
	}
	
	public static <T> int computeDeepHashCode(Ref<T> obj) {
		if( obj == null ) {
			return 0;
		}
		return obj.deepHashCode();
	}
	
	public default boolean deepEquals(Ref<T> obj) {
		if( obj == null ) {
			return false;
		}
		return Objects.equals(_type(), obj._type())
				&& Objects.equals(id(), obj.id())
				&& Objects.equals(etag(), obj.etag());
	}
	
	public static <T> boolean computeDeepEquals(Ref<T> a, Ref<T> b) {
		if( a == b ) {
			return true;
		} else if( a == null ) {
			return false;
		} else {
			return a.deepEquals(b);
		}
	}
	
	public static <T> Ref<T> of(Class<T> type, String id) {
		return new RefImpl<>(type, id);
	}
	
	public static <T> Ref<T> of(Class<T> type, String id, String etag) {
		return new RefImpl<>(type, id, etag);
	}
	
	public static String toString(Ref<?> ref) {
		if( ref == null ) {
			return null;
		}
		return ref._type().getSimpleName() + "(" + asString(ref) + ")";
	}
	
	public static String asString(Ref<?> ref) {
	    if( ref.etag() == null ) {
	        return ref.id();
	    }
		return ref.id() + "@" + ref.etag();
	}
	
	public static <T> Ref<T> fromString(Class<T> clazz, String value) {
		int idx = value.indexOf('@');
		if( idx < 0 ) {
			return Ref.of(clazz, value);
		} else {
			return Ref.of(clazz, value.substring(0, idx), value.substring(idx+1));
		}
	}
}
