package at.bestsolution.rsd.rt;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;

import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

@Component
public class SerializationRegistry {
	private final Map<Class<?>, List<Serializer<?>>> typeSerializers = Collections.synchronizedMap(new HashMap<>());
	private final Map<Class<?>, List<Deserializer<?>>> typeDeserializers = Collections.synchronizedMap(new HashMap<>());

	@Reference(cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY)
	public void registerSerializer(Serializer<?> serializer) {
		this.typeSerializers.computeIfAbsent(serializer.type(), v -> new CopyOnWriteArrayList<>()).add(serializer);
	}

	public void unregisterSerializer(Serializer<?> serializer) {
		List<Serializer<?>> list = this.typeSerializers.get(serializer.type());
		if (list != null) {
			list.remove(serializer);
		}
	}

	@Reference(cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY)
	public void registerDeserializer(Deserializer<?> serializer) {
		this.typeDeserializers.computeIfAbsent(serializer.type(), v -> new CopyOnWriteArrayList<>()).add(serializer);
	}

	public void unregisterDeserializer(Deserializer<?> serializer) {
		List<Deserializer<?>> list = this.typeDeserializers.get(serializer.type());
		if (list != null) {
			list.remove(serializer);
		}
	}
}
