package at.bestsolution.rsd.rt.delta;

import at.bestsolution.rsd.rt.util.BooleanConsumer;

public abstract class BooleanValueImpl extends ValueImpl<Boolean> implements BooleanValue {

	static BooleanValue TRUE = new TrueBooleanValueImpl();
	static BooleanValue FALSE = new FalseBooleanValueImpl();
	static BooleanValue NULL = new NullBooleanValueImpl();
	static BooleanValue UNDEFINED = new UndefinedBooleanValueImpl();

	@Override
	public boolean booleanValue() {
		return getValue().booleanValue();
	}

	@Override
	public boolean isNull() {
		return false;
	}

	@Override
	public boolean isUndefined() {
		return false;
	}

	static final class TrueBooleanValueImpl extends BooleanValueImpl {

		@Override
		protected Boolean getValue() {
			return Boolean.TRUE;
		}

		@Override
		public void ifDefined( boolean nullValue, BooleanConsumer action ) {
			action.accept( true );
		}

		@Override
		public void ifPresent( BooleanConsumer action ) {
			action.accept( true );
		}
	}

	static final class FalseBooleanValueImpl extends BooleanValueImpl {

		@Override
		protected Boolean getValue() {
			return Boolean.FALSE;
		}

		@Override
		public void ifDefined( boolean nullValue, BooleanConsumer action ) {
			action.accept( false );
		}

		@Override
		public void ifPresent( BooleanConsumer action ) {
			action.accept( false );
		}
	}

	static final class UndefinedBooleanValueImpl extends UndefinedValueImpl<Boolean> implements BooleanValue {

		@Override
		public boolean booleanValue() {
			throw new IllegalStateException( "Value is undefined" );
		}

		@Override
		public void ifDefined( boolean nullValue, BooleanConsumer action ) {

		}

		@Override
		public void ifPresent( BooleanConsumer action ) {

		}
	}

	static final class NullBooleanValueImpl extends NullValueImpl<Boolean> implements BooleanValue {

		@Override
		public boolean booleanValue() {
			throw new IllegalStateException( "Value is NULL" );
		}

		@Override
		public void ifDefined( boolean nullValue, BooleanConsumer action ) {
			action.accept( nullValue );
		}

		@Override
		public void ifPresent( BooleanConsumer action ) {

		}
	}
}
