package at.bestsolution.rsd.rt.delta;

import java.util.function.DoubleConsumer;
import java.util.function.DoubleFunction;
import java.util.function.DoubleUnaryOperator;

public final class DoubleValueImpl extends ValueImpl<Double> implements DoubleValue {
    private final double value;
    
    static final DoubleValue UNDEFINED = new UndefinedDoubleValueImpl();
    static final DoubleValue NULL = new NullDoubleValueImpl();
    
    public static DoubleValue of(double value) {
        return new DoubleValueImpl(value);
    }
    
    private DoubleValueImpl(double value) {
        this.value = value;
    }

    @Override
    public double doubleValue() {
        return value;
    }

    @Override
    public DoubleValue map(DoubleUnaryOperator mapper) {
        return DoubleValue.of(mapper.applyAsDouble(value));
    }
    
    @Override
    public <T> Value<T> mapToObj(DoubleFunction<? extends T> mapper) {
        T rv = mapper.apply(value);
        return rv == null ? Value.NULL() : Value.of(rv);
    }

    @Override
    protected Double getValue() {
        return value;
    }

    @Override
    public boolean isNull() {
        return false;
    }
    
    @Override
    public boolean isUndefined() {
        return false;
    }
    
    @Override
    public void ifPresent( DoubleConsumer action ) {
    	action.accept( value );
    }
    
    @Override
    public void ifDefined( double nullValue, DoubleConsumer action ) {
    	action.accept( value );
    }
    
    static class UndefinedDoubleValueImpl extends UndefinedValueImpl<Double> implements DoubleValue {

        @Override
        public double doubleValue() {
            throw new IllegalStateException("Value is undefined");
        }

        @Override
        public DoubleValue map(DoubleUnaryOperator mapper) {
            return this;
        }

        @Override
        public <T> Value<T> mapToObj(DoubleFunction<? extends T> mapper) {
            return Value.UNDEFINED();
        }
        
        @Override
        public void ifPresent( DoubleConsumer action ) {
        	
        }
        
        @Override
        public void ifDefined( double nullValue, DoubleConsumer action ) {
        	
        }
    }
    
    static class NullDoubleValueImpl extends NullValueImpl<Double> implements DoubleValue {

        @Override
        public double doubleValue() {
            throw new IllegalStateException("Value is null");
        }

        @Override
        public DoubleValue map(DoubleUnaryOperator mapper) {
            return this;
        }

        @Override
        public <T> Value<T> mapToObj(DoubleFunction<? extends T> mapper) {
            return Value.NULL();
        }
        
        @Override
        public void ifPresent( DoubleConsumer action ) {
        	
        }
        
        @Override
        public void ifDefined( double nullValue, DoubleConsumer action ) {
        	action.accept( nullValue );
        }
    }
}
