package at.bestsolution.rsd.rt.delta;

import java.util.function.IntConsumer;
import java.util.function.IntFunction;
import java.util.function.IntUnaryOperator;

public final class IntValueImpl extends ValueImpl<Integer> implements IntValue {
    private int value;
    
    static final IntValue UNDEFINED = new UndefinedIntValueImpl();
    static final IntValue NULL = new NullIntValueImpl();
    
    static IntValue of(int value) {
        return new IntValueImpl(value);
    }
    
    private IntValueImpl(int value) {
        this.value = value;
    }

    @Override
    public int intValue() {
        return value;
    }

    @Override
    public IntValue map(IntUnaryOperator mapper) {
        return IntValue.of(mapper.applyAsInt(value));
    }
    
    @Override
    public <T> Value<T> mapToObj(IntFunction<? extends T> mapper) {
        T rv = mapper.apply(value);
        return rv == null ? Value.NULL() : Value.of(rv);
    }

    @Override
    protected Integer getValue() {
        return value;
    }

    @Override
    public boolean isNull() {
        return false;
    }
    
    @Override
    public boolean isUndefined() {
        return false;
    }
    
	public void ifPresent(IntConsumer action) {
		action.accept( value );
	}
    
    public void ifDefined(int nullValue, IntConsumer action) {
    	action.accept( value );
    }
    
    static class UndefinedIntValueImpl extends UndefinedValueImpl<Integer> implements IntValue {

        @Override
        public int intValue() {
            throw new IllegalStateException("Value is undefined");
        }

        @Override
        public IntValue map(IntUnaryOperator mapper) {
            return this;
        }

        @Override
        public <T> Value<T> mapToObj(IntFunction<? extends T> mapper) {
            return Value.UNDEFINED();
        }
        
    	public void ifPresent(IntConsumer action) {
    		
    	}
        
        public void ifDefined(int nullValue, IntConsumer action) {
        	
        }
    }
    
    static class NullIntValueImpl extends NullValueImpl<Integer> implements IntValue {

        @Override
        public int intValue() {
            throw new IllegalStateException("Value is null");
        }

        @Override
        public IntValue map(IntUnaryOperator mapper) {
            return this;
        }

        @Override
        public <T> Value<T> mapToObj(IntFunction<? extends T> mapper) {
            return Value.NULL();
        }
        
    	public void ifPresent(IntConsumer action) {
    		
    	}
        
        public void ifDefined(int nullValue, IntConsumer action) {
        	action.accept( nullValue );
        }
    }
}
