package at.bestsolution.rsd.rt.delta;

import java.util.ArrayList;
import java.util.List;
import java.util.OptionalInt;

import at.bestsolution.rsd.rt.Array;

public interface ListChange<O,K> {
    @SuppressWarnings("unchecked")
    public static <O,K> ListChange<O, K> EMPTY() {
        return ListChangeImpl.EMPTY;
    }
    
    @SafeVarargs
    public static <O,K> ListChange<O,K> ofAdditions(O... elements) {
        return new Builder<O,K>().append(elements).build();
    }
    
    @SafeVarargs
    public static <O,K> ListChange<O,K> ofRemovals(K... elements) {
        return new Builder<O,K>().remove(elements).build();
    }
    
    public Array<Add<O>> additions();
    public Array<Remove<K>> removals();
    
    public boolean isEmpty();
    
    public interface Add<T> {
        OptionalInt index();
        Array<T> elements();
    }
    
    public interface Remove<T> {
        OptionalInt index();
        Array<T> elements();
    }
    
    public static class Builder<O,K> {
        private List<Add<O>> additions = new ArrayList<>();
        private List<Remove<K>> removals = new ArrayList<>();
        
        @SafeVarargs
        public final Builder<O, K> insert(int idx, O... elements) {
            additions.add(new ListChangeImpl.AddImpl<>(idx, elements));
            return this;
        }
        
        public final Builder<O, K> insert(int idx, Array<O> elements) {
            additions.add(new ListChangeImpl.AddImpl<>(idx, elements));
            return this;
        }
        
        @SafeVarargs
        public final Builder<O, K> append(O... elements) {
            additions.add(new ListChangeImpl.AddImpl<>(-1, elements));
            return this;
        }

        public final Builder<O, K> append(Array<O> elements) {
            additions.add(new ListChangeImpl.AddImpl<>(-1, elements));
            return this;
        }

        @SafeVarargs
        public final Builder<O, K> remove(K... elements) {
            removals.add(new ListChangeImpl.RemoveImpl<>(-1, elements));
            return this;
        }
        
        public final Builder<O, K> remove(Array<K> elements) {
            removals.add(new ListChangeImpl.RemoveImpl<>(-1, elements));
            return this;
        }

        @SafeVarargs
        public final Builder<O, K> remove(int index, K... elements) {
            removals.add(new ListChangeImpl.RemoveImpl<>(-1, elements));
            return this;
        }
        
        public final Builder<O, K> remove(int index, Array<K> elements) {
            removals.add(new ListChangeImpl.RemoveImpl<>(-1, elements));
            return this;
        }

        public ListChange<O, K> build() {
            return new ListChangeImpl<>(Array.of(additions), Array.of(removals));
        }
    }
}
