package at.bestsolution.rsd.rt.delta;

import java.io.Serializable;
import java.util.OptionalInt;

import at.bestsolution.rsd.rt.Array;

class ListChangeImpl<O,K> implements ListChange<O, K>, Serializable {

    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    @SuppressWarnings({ "rawtypes", "unchecked" })
    static final ListChangeImpl EMPTY = new ListChangeImpl(Array.empty(), Array.empty());
    
    private final Array<Add<O>> additions;
    
    private final Array<Remove<K>> removals;
    
    public ListChangeImpl(Array<Add<O>> additions, Array<Remove<K>> removals) {
        this.additions = additions;
        this.removals = removals;
    }
    
    @Override
    public boolean isEmpty() {
    	return additions.isEmpty() && removals.isEmpty();
    }
    
    @Override
    public Array<Add<O>> additions() {
        return this.additions;
    }

    @Override
    public Array<Remove<K>> removals() {
        return this.removals;
    }
    
    @Override
    public String toString() {
        if( additions.isEmpty() && removals.isEmpty() ) {
            return "ListChange(empty)";
        }
        return String.format("ListChange(additions: %s, removals: %s)", additions, removals);
    }

    public static class AddImpl<O> implements Add<O>, Serializable {
        /**
         * 
         */
        private static final long serialVersionUID = 1L;

        private final int index;
        private final Array<O> elements;
        
        public AddImpl(int index, O[] elements) {
            this.index = index;
            this.elements = Array.of(elements);
        }
        
        public AddImpl(int index, Array<O> elements) {
            this.index = index;
            this.elements = elements;
        }
        
        @Override
        public Array<O> elements() {
            return elements;
        }
        
        @Override
        public OptionalInt index() {
            return index < 0 ? OptionalInt.empty() : OptionalInt.of(index);
        }
    }
    
    public static class RemoveImpl<O> implements Remove<O>, Serializable {
        /**
         * 
         */
        private static final long serialVersionUID = 1L;

        private final int index;
        private final Array<O> elements;

        public RemoveImpl(int index, O[] elements) {
            this.index = index;
            this.elements = Array.of(elements);
        }
        
        public RemoveImpl(int index, Array<O> elements) {
            this.index = index;
            this.elements = elements;
        }
        
        @Override
        public Array<O> elements() {
            return elements;
        }
        
        @Override
        public OptionalInt index() {
            return index < 0 ? OptionalInt.empty() : OptionalInt.of(index);
        }
        
    }
}
