package at.bestsolution.rsd.rt.delta;

import java.util.function.LongConsumer;
import java.util.function.LongFunction;
import java.util.function.LongUnaryOperator;

public final class LongValueImpl extends ValueImpl<Long> implements LongValue {
    private final long value;

    static final LongValue UNDEFINED = new UndefinedLongValueImpl();
    static final LongValue NULL = new NullLongValueImpl();

    public static LongValue of(long value) {
        return new LongValueImpl(value);
    }

    private LongValueImpl(long value) {
        this.value = value;
    }

    @Override
    public long longValue() {
        return value;
    }

    @Override
    public LongValue map(LongUnaryOperator mapper) {
        return LongValue.of(mapper.applyAsLong(value));
    }

    @Override
    public <T> Value<T> mapToObj(LongFunction<? extends T> mapper) {
        T rv = mapper.apply(value);
        return rv == null ? Value.NULL() : Value.of(rv);
    }

    @Override
    protected Long getValue() {
        return value;
    }

    @Override
    public boolean isNull() {
        return false;
    }

    @Override
    public boolean isUndefined() {
        return false;
    }
    
    @Override
    public void ifDefined( long nullValue, LongConsumer action ) {
    	action.accept( value );
    }
    
    @Override
    public void ifPresent( LongConsumer action ) {
    	action.accept( value );
    }

    static class UndefinedLongValueImpl extends UndefinedValueImpl<Long> implements LongValue {

        @Override
        public long longValue() {
            throw new IllegalStateException("Value is undefined");
        }

        @Override
        public LongValue map(LongUnaryOperator mapper) {
            return this;
        }

        @Override
        public <T> Value<T> mapToObj(LongFunction<? extends T> mapper) {
            return Value.UNDEFINED();
        }
        
        @Override
        public void ifDefined( long nullValue, LongConsumer action ) {
        	
        }
        
        @Override
        public void ifPresent( LongConsumer action ) {
        	
        }
    }

    static class NullLongValueImpl extends NullValueImpl<Long> implements LongValue {

        @Override
        public long longValue() {
            throw new IllegalStateException("Value is null");
        }

        @Override
        public LongValue map(LongUnaryOperator mapper) {
            return this;
        }

        @Override
        public <T> Value<T> mapToObj(LongFunction<? extends T> mapper) {
            return Value.NULL();
        }

        @Override
        public void ifDefined( long nullValue, LongConsumer action ) {
        	action.accept( nullValue );
        }
        
        @Override
        public void ifPresent( LongConsumer action ) {
        	
        }
    }
}
