package at.bestsolution.rsd.rt.delta;

import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.ToIntFunction;

abstract class ValueImpl<T> implements Value<T> {

    @SuppressWarnings({ "rawtypes" })
    static final Value UNDEFINED = new UndefinedValueImpl();
    
    @SuppressWarnings({ "rawtypes" })
    static final Value NULL = new NullValueImpl();
    
    protected abstract T getValue();
    
    public <U> Value<U> map(Function<T, U> map) {
        if (isUndefined()) {
            return Value.UNDEFINED();
        } else if( isNull() ) {
            return Value.NULL();
        }
        return Value.of(map.apply(value()));
    }
    
    @Override
    public IntValue mapToInt(ToIntFunction<T> mapper) {
        if (isUndefined()) {
            return IntValue.UNDEFINED();
        } else if( isNull() ) {
            return IntValue.NULL();
        }
        return IntValue.of(mapper.applyAsInt(getValue()));
    }
    
    @Override
    public void ifPresent(Consumer<T> action) {
        if( isDefined() && isNotNull() ) {
            action.accept(getValue());
        }
    }
    
    @Override
    public void ifDefined(Consumer<T> action) {
        if( isDefined() ) {
            action.accept(getValue());
        }
    }

    @Override
    public T value() {
        if( isUndefined() ) {
            throw new IllegalStateException("Value is undefined");
        }
        return getValue();
    }
    
    
    @Override
    public boolean isNotNull() {
        return ! isNull();
    }

    @Override
    public boolean isDefined() {
        return ! isUndefined();
    }
    
    public String toString() {
        if( isUndefined() ) {
            return "Value(undefined)";
        } else if( isNull() ) {
            return "Value(null)";
        } else {
            return String.format("Value(%s)", getValue());
        }
    }
    
    static class UndefinedValueImpl<T> extends ValueImpl<T> {

        @Override
        public final boolean isUndefined() {
            return true;
        }

        @Override
        public final boolean isNull() {
            return false;
        }

        @Override
        protected final T getValue() {
            throw new IllegalStateException("Value is undefined");
        }
    }
    
    static class NullValueImpl<T> extends ValueImpl<T> {

        @Override
        public final boolean isUndefined() {
            return false;
        }
        
        @Override
        public final boolean isNull() {
            return true;
        }
        
        @Override
        protected final T getValue() {
            return null;
        }
    }
}
