package at.bestsolution.rsd.rt.interceptor;

import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.function.Supplier;

import at.bestsolution.rsd.rt.Array;

public interface MethodInvocationDescriptor {
	public String name();
	
	public Class<?> instanceType();
	public <S> S instance();
	
	public Array<Parameter> parameters();
	
	public <T extends Throwable> T exception();
	
	public Type returnType();
	
	public <O> O returnValue();
	
	public interface Parameter {
		public Class<?> type();
//		public Type genericType();
		public String name();
		public <O> O value();
		
		public static Parameter create(String name, Class<?> type, Object value) {
			class LocalParameterImpl implements Parameter {

				@Override
				public Class<?> type() {
					return type;
				}

				@Override
				public String name() {
					return name;
				}

				@SuppressWarnings("unchecked")
				@Override
				public <O> O value() {
					return (O) value;
				}
				
			}
			
			return new LocalParameterImpl();
		}
	}
	
	public static MethodInvocationDescriptor create(Class<?> instanceType, Object instance, String methodName, Throwable exception, Object returnValue, Supplier<Method> methodSupplier, Object[] parameterValues) {
		return new MethodInvocationDescriptorImpl(instanceType, instance, methodName, parameterValues, exception, returnValue, methodSupplier);
	}
	
	public static MethodInvocationDescriptor createVoid(Class<?> instanceType,Object instance, String methodName, Throwable exception, Supplier<Array<Parameter>> parameters) {
		return create(instanceType, instance, methodName, exception, Void.class, null, parameters);
	}
	
	public static MethodInvocationDescriptor createVoidNoParam(Class<?> instanceType,Object instance, String methodName, Throwable exception) {
		return create(instanceType, instance, methodName, exception, Void.class, null, Array::empty);
	}
	
	public static MethodInvocationDescriptor createNoParam(Class<?> instanceType,Object instance, String methodName, Throwable exception, Class<?> returnType, Object returnValue) {
		return create(instanceType, instance, methodName, exception, returnType, returnValue, Array::empty);
	}
	
	public static MethodInvocationDescriptor create(Class<?> instanceType,Object instance, String methodName, Throwable exception, Class<?> returnType, Object returnValue, Supplier<Array<Parameter>> parameters) {
		class LocalMethodInvocationDescriptorImpl implements MethodInvocationDescriptor {
			private Array<Parameter> cachedParameters;
			
			@Override
			public String name() {
				return methodName;
			}
			
			@Override
			public Class<?> instanceType() {
				return instanceType;
			}

			@SuppressWarnings("unchecked")
			@Override
			public <S> S instance() {
				return (S) instance;
			}

			@Override
			public Array<Parameter> parameters() {
				if( cachedParameters == null ) {
					cachedParameters = parameters.get();
				}
				return cachedParameters;
			}

			@SuppressWarnings("unchecked")
			@Override
			public <T extends Throwable> T exception() {
				return (T) exception;
			}

			@Override
			public Type returnType() {
				return returnType;
			}

			@SuppressWarnings("unchecked")
			@Override
			public <O> O returnValue() {
				return (O) returnValue;
			}
		}
		return new LocalMethodInvocationDescriptorImpl();
	}
}
