package at.bestsolution.rsd.rt.interceptor;

import java.lang.reflect.Method;
import java.util.function.Supplier;

import at.bestsolution.rsd.rt.Array;

final class MethodInvocationDescriptorImpl implements MethodInvocationDescriptor {
	private final String methodName;
	private final Class<?> instanceType;
	private final Object instance;
	private final Throwable exception;
	private final Object returnValue;
	private final Supplier<Method> methodSupplier;
	private final Object[] parameterValues;
	
	// Populate lazy
	private Array<Parameter> parameters;
	private Method cachedMethod;
	
	public MethodInvocationDescriptorImpl(Class<?> instanceType, Object instance, String methodName, Object[] parameterValues,
			Throwable exception, Object returnValue, Supplier<Method> methodSupplier) {
		this.instanceType = instanceType;
		this.instance = instance;
		this.methodName = methodName;
		this.parameterValues = parameterValues;
		this.exception = exception;
		this.returnValue = returnValue;
		this.methodSupplier = methodSupplier;
	}
	
	private Method cachedMethod() {
		if( cachedMethod == null ) {
			cachedMethod = methodSupplier.get();
		}
		return cachedMethod;
	}
	
	@Override
	public Class<?> instanceType() {
		return instanceType;
	}

	@Override
	public String name() {
		return this.methodName;
	}

	@SuppressWarnings("unchecked")
	@Override
	public <S> S instance() {
		return (S) instance;
	}

	@Override
	public Array<Parameter> parameters() {
		if( parameters == null ) {
			Parameter[] parameters = new Parameter[parameterValues.length];
			for( int i = 0; i < parameters.length; i++ ) {
				parameters[i] = new ParameterImpl(this::cachedMethod, i, parameterValues[i]);
			}
			this.parameters = Array.of(parameters);
		}
		return this.parameters;
	}

	@SuppressWarnings("unchecked")
	@Override
	public <T extends Throwable> T exception() {
		return (T) exception;
	}

	@Override
	public Class<?> returnType() {
		return methodSupplier.get().getReturnType();
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public <O> O returnValue() {
		return (O) returnValue;
	}

	final static class ParameterImpl implements Parameter {
		private final Supplier<Method> methodSupplier;
		private final Object value;
		private final int index;
		
		public ParameterImpl(Supplier<Method> methodSupplier, int index, Object value) {
			this.methodSupplier = methodSupplier;
			this.value = value;
			this.index = index;
		}

		@Override
		public Class<?> type() {
			return methodSupplier.get().getParameterTypes()[index];
		}
		
		@Override
		public String name() {
			return methodSupplier.get().getParameters()[index].getName();
		}

		@SuppressWarnings("unchecked")
		@Override
		public <O> O value() {
			return (O) value;
		}
		
	}
}
